@@ -26,12 +26,13 @@ import json
 import logging
 import math
 import os
+from pathlib import Path
 from typing import Generator
 
 import numpy as np
-import pyproj
 import shapefile
 from PIL import Image
+from pyproj import CRS, Proj
 
 try:
     from pyresample import AreaDefinition
@@ -45,16 +46,11 @@ def get_resolution_from_area(area_def):
     """Get the best resolution for an area definition."""
     x_size = area_def.width
     y_size = area_def.height
-    prj = Proj(area_def.crs if hasattr(area_def, "crs") else area_def.proj_str)
-    if prj.is_latlong():
-        x_ll, y_ll = prj(area_def.area_extent[0], area_def.area_extent[1])
-        x_ur, y_ur = prj(area_def.area_extent[2], area_def.area_extent[3])
-        x_resolution = (x_ur - x_ll) / x_size
-        y_resolution = (y_ur - y_ll) / y_size
-    else:
-        x_resolution = (area_def.area_extent[2] - area_def.area_extent[0]) / x_size
-        y_resolution = (area_def.area_extent[3] - area_def.area_extent[1]) / y_size
+    x_resolution = abs(area_def.area_extent[2] - area_def.area_extent[0]) / x_size
+    y_resolution = abs(area_def.area_extent[3] - area_def.area_extent[1]) / y_size
     res = min(x_resolution, y_resolution)
+    if "degree" in area_def.crs.axis_info[0].unit_name:
+        res = _estimate_metered_resolution_from_degrees(area_def.crs, res)
 
     if res > 25000:
         return "c"
@@ -68,6 +64,12 @@ def get_resolution_from_area(area_def):
         return "f"
 
 
+def _estimate_metered_resolution_from_degrees(crs: CRS, resolution_degrees: float) -> float:
+    major_radius = crs.datum.ellipsoid.semi_major_metre
+    # estimate by taking the arc length using the radius
+    return major_radius * math.radians(resolution_degrees)
+
+
 class _CoordConverter:
     """Convert coordinates from one space to in-bound image pixel column and row.
 
@@ -91,7 +93,8 @@ class _CoordConverter:
             raise ValueError(f"'coord_ref' must be one of {pretty_coord_refs}.")
         self._convert_method = convert_methods[coord_ref]
 
-    def _check_area_def(self, area_def):
+    @staticmethod
+    def _check_area_def(area_def):
         if AreaDefinition is None:
             raise ImportError(
                 "Missing required 'pyresample' module, please "
@@ -130,16 +133,6 @@ def hash_dict(dict_to_hash: dict) -> str:
     return dhash.hexdigest()
 
 
-class Proj(pyproj.Proj):
-    """Wrapper around pyproj to add in 'is_latlong'."""
-
-    def is_latlong(self):
-        if hasattr(self, "crs"):
-            return self.crs.is_geographic
-        # pyproj<2.0
-        return super(Proj, self).is_latlong()
-
-
 class ContourWriterBase(object):
     """Base class for contourwriters. Do not instantiate.
 
@@ -167,7 +160,16 @@ class ContourWriterBase(object):
 
     def _draw_text(self, draw, position, txt, font, align="cc", **kwargs):
         """Draw text with agg module."""
-        txt_width, txt_height = draw.textsize(txt, font)
+        if hasattr(draw, "textsize"):
+            txt_width, txt_height = draw.textsize(txt, font)
+        else:
+            left, top, right, bottom = draw.textbbox(position, txt, font)
+            # bbox is based on "left ascender" anchor for horizontal text
+            # but does not include the ascender to top distance.
+            # In order to include that additional distance we take height from
+            # anchor (`position`) to the bottom of the text. See:
+            # https://pillow.readthedocs.io/en/stable/handbook/text-anchors.html#text-anchors
+            txt_width, txt_height = right - left, bottom - position[1]
         x_pos, y_pos = position
         ax, ay = align.lower()
         if ax == "r":
@@ -178,7 +180,7 @@ class ContourWriterBase(object):
         if ay == "b":
             y_pos = y_pos - txt_height
         elif ay == "c":
-            y_pos = y_pos - txt_width / 2
+            y_pos = y_pos - txt_height / 2
 
         self._engine_text_draw(draw, x_pos, y_pos, txt, font, **kwargs)
 
@@ -500,16 +502,21 @@ class ContourWriterBase(object):
         return overlays
 
     def add_overlay_from_dict(self, overlays, area_def, cache_epoch=None, background=None):
-        """Create and return a transparent image adding all the overlays contained in the `overlays` dict.
+        """Create and return a transparent image adding all the overlays contained in the ``overlays`` dict.
 
         Optionally caches overlay results for faster rendering of images with
         the same provided AreaDefinition and parameters. Cached results are
-        identified by hashing the AreaDefinition and the overlays dictionary.
+        identified by hashing the AreaDefinition and the ``overlays`` dictionary.
+
+        Note that if ``background`` is provided and caching is not used, the
+        result will be the final result of applying the overlays onto the
+        background. This is due to an optimization step avoiding creating a
+        separate overlay image in memory when it isn't needed.
 
         .. warning::
 
             Font objects are ignored in parameter hashing as they can't be easily hashed.
-            Therefore font changes will not trigger a new rendering for the cache.
+            Therefore, font changes will not trigger a new rendering for the cache.
 
         :Parameters:
             overlays : dict
@@ -524,24 +531,33 @@ class ContourWriterBase(object):
                 provided dictionary (see below).
             background: pillow image instance
                 The image on which to write the overlays on. If it's None (default),
-                a new image is created, otherwise the provide background is
+                a new image is created, otherwise the provided background is
                 used and changed *in place*.
 
+        The keys in ``overlays`` that will be taken into account are:
+        cache, coasts, rivers, borders, shapefiles, grid, cities, points
 
-            The keys in `overlays` that will be taken into account are:
-            cache, coasts, rivers, borders, shapefiles, grid, cities, points
+        For all of them except ``cache``, the items are the same as the
+        corresponding functions in pycoast, so refer to the docstrings of
+        these functions (add_coastlines, add_rivers, add_borders,
+        add_shapefile_shapes, add_grid, add_cities, add_points).
+        For cache, two parameters are configurable:
 
-            For all of them except `cache`, the items are the same as the
-            corresponding functions in pycoast, so refer to the docstrings of
-            these functions (add_coastlines, add_rivers, add_borders,
-            add_shapefile_shapes, add_grid, add_cities, add_points).
-            For cache, two parameters are configurable:
+        - `file`:
+            specify the directory and the prefix
+            of the file to save the caches decoration to (for example
+            /var/run/black_coasts_red_borders)
+        - `regenerate`:
+            True or False (default) to force the overwriting
+            of an already cached file.
 
-            - `file`: specify the directory and the prefix
-                  of the file to save the caches decoration to (for example
-                  /var/run/black_coasts_red_borders)
-            - `regenerate`: True or False (default) to force the overwriting
-                  of an already cached file.
+        :Returns: PIL.Image.Image
+
+            Resulting overlays as an Image object. If caching was used then
+            the Image wraps an open file and should be closed by the caller.
+            If caching was not used or the cached image was recreated then
+            this is an in-memory Image object. Regardless, it can be closed
+            by calling the ``.close()`` method of the Image.
 
         """
         overlay_helper = _OverlaysFromDict(self, overlays, area_def, cache_epoch, background)
@@ -550,12 +566,22 @@ class ContourWriterBase(object):
     def add_overlay_from_config(self, config_file, area_def, background=None):
         """Create and return a transparent image adding all the overlays contained in a configuration file.
 
+        See :meth:`add_overlay_from_dict` for more information.
+
         :Parameters:
             config_file : str
                 Configuration file name
             area_def : object
                 Area Definition of the creating image
 
+        :Returns: PIL.Image.Image
+
+            Resulting overlays as an Image object. If caching was used then
+            the Image wraps an open file and should be closed by the caller.
+            If caching was not used or the cached image was recreated then
+            this is an in-memory Image object. Regardless, it can be closed
+            by calling the ``.close()`` method of the Image.
+
         """
         overlays = self._config_to_dict(config_file)
         return self.add_overlay_from_dict(overlays, area_def, os.path.getmtime(config_file), background)
@@ -699,8 +725,7 @@ class ContourWriterBase(object):
         # cities.red is a reduced version of the files avalable at http://download.geonames.org
         # Fields: 0=name (UTF-8), 1=asciiname, 2=longitude [°E], 3=latitude [°N], 4=countrycode
         cities_filename = os.path.join(db_root_path, os.path.join("CITIES", "cities.txt"))
-        cities_parser = GeoNamesCitiesParser(cities_filename)
-        for city_name, lon, lat in cities_parser.iter_cities_names_lon_lat(cities_list):
+        for city_name, lon, lat in iter_cities_names_lon_lat(cities_filename, cities_list):
             try:
                 x, y = area_def.get_array_indices_from_lonlat(lon, lat)
             except ValueError:
@@ -1053,16 +1078,10 @@ def _get_bounding_box_lonlat_sides(area_extent, x_size, y_size, prj):
     x_range = np.linspace(x_ll, x_ur, num=x_size)
     y_range = np.linspace(y_ll, y_ur, num=y_size)
 
-    if prj.is_latlong():
-        lons_s1, lats_s1 = x_ll * np.ones(y_range.size), y_range
-        lons_s2, lats_s2 = x_range, y_ur * np.ones(x_range.size)
-        lons_s3, lats_s3 = x_ur * np.ones(y_range.size), y_range
-        lons_s4, lats_s4 = x_range, y_ll * np.ones(x_range.size)
-    else:
-        lons_s1, lats_s1 = prj(np.ones(y_range.size) * x_ll, y_range, inverse=True)
-        lons_s2, lats_s2 = prj(x_range, np.ones(x_range.size) * y_ur, inverse=True)
-        lons_s3, lats_s3 = prj(np.ones(y_range.size) * x_ur, y_range, inverse=True)
-        lons_s4, lats_s4 = prj(x_range, np.ones(x_range.size) * y_ll, inverse=True)
+    lons_s1, lats_s1 = prj(np.ones(y_range.size) * x_ll, y_range, inverse=True)
+    lons_s2, lats_s2 = prj(x_range, np.ones(x_range.size) * y_ur, inverse=True)
+    lons_s3, lats_s3 = prj(np.ones(y_range.size) * x_ur, y_range, inverse=True)
+    lons_s4, lats_s4 = prj(x_range, np.ones(x_range.size) * y_ll, inverse=True)
     return (lons_s1, lons_s2, lons_s3, lons_s4), (lats_s1, lats_s2, lats_s3, lats_s4)
 
 
@@ -1087,12 +1106,7 @@ def _get_pixel_index(shape, area_extent, x_size, y_size, prj, x_offset=0, y_offs
     shape_data = np.array(shape.points if hasattr(shape, "points") else shape)
     lons = shape_data[:, 0]
     lats = shape_data[:, 1]
-
-    if prj.is_latlong():
-        x_ll, y_ll = prj(area_extent[0], area_extent[1])
-        x_ur, y_ur = prj(area_extent[2], area_extent[3])
-    else:
-        x_ll, y_ll, x_ur, y_ur = area_extent
+    x_ll, y_ll, x_ur, y_ur = area_extent
 
     x, y = prj(lons, lats)
 
@@ -1137,14 +1151,12 @@ def _get_pixel_index(shape, area_extent, x_size, y_size, prj, x_offset=0, y_offs
     return index_arrays, is_reduced
 
 
-class GeoNamesCitiesParser:
-    """Helper for parsing citiesN.txt files from GeoNames.org."""
-
-    def __init__(self, cities_filename: str):
-        self._cities_file = open(cities_filename, mode="r", encoding="utf-8")
-
-    def iter_cities_names_lon_lat(self, cities_list: list[str]) -> Generator[tuple[str, float, float], None, None]:
-        for city_row in self._cities_file:
+def iter_cities_names_lon_lat(
+    cities_filename: str | Path, cities_list: list[str]
+) -> Generator[tuple[str, float, float], None, None]:
+    """Iterate over citiesN.txt files from GeoNames.org."""
+    with open(cities_filename, mode="r", encoding="utf-8") as cities_file:
+        for city_row in cities_file:
             city_info = city_row.split("\t")
             if not city_info or not (city_info[1] in cities_list or city_info[2] in cities_list):
                 continue
@@ -1199,7 +1211,7 @@ class _OverlaysFromDict:
                 foreground = Image.open(cache_file)
                 logger.info("Using image in cache %s", cache_file)
                 if background is not None:
-                    background.paste(foreground, mask=foreground.split()[-1])
+                    _apply_cached_foreground_on_background(background, foreground)
                 return foreground
             logger.info("Regenerating cache file.")
         except OSError:
@@ -1212,7 +1224,7 @@ class _OverlaysFromDict:
         except IOError as e:
             logger.error("Can't save cache: %s", str(e))
         if self._background is not None:
-            self._background.paste(self._foreground, mask=self._foreground.split()[-1])
+            _apply_cached_foreground_on_background(self._background, self._foreground)
 
     def _generate_cache_filename(self, cache_prefix, area_def, overlays_dict):
         area_hash = hash(area_def)
@@ -1443,6 +1455,18 @@ class _OverlaysFromDict:
         )
 
 
+def _apply_cached_foreground_on_background(background, foreground):
+    premult_foreground = foreground.convert("RGBa")
+    if background.mode == "RGBA":
+        # Cached foreground and background are both RGBA, no extra conversions needed
+        background.paste(premult_foreground, mask=premult_foreground)
+        return
+    background_rgba = background.convert("RGBA")
+    background_rgba.paste(premult_foreground, mask=premult_foreground)
+    # overwrite background image in place
+    background.paste(background_rgba)
+
+
 class _GridDrawer:
     """Helper for drawing graticule/grid lines."""
 
@@ -1514,7 +1538,7 @@ class _GridDrawer:
         min_lons[min_lons > 180] = min_lons[min_lons > 180] - 360
 
         # Get min_lons not in maj_lons
-        min_lons = np.lib.arraysetops.setdiff1d(min_lons, maj_lons)
+        min_lons = np.setdiff1d(min_lons, maj_lons)
 
         # lats along major lon lines
         lin_lats = np.arange(
@@ -1535,7 +1559,7 @@ class _GridDrawer:
         min_lats = np.arange(round_lat_min + increase_min_lat, lat_max - shorten_max_lat, dlat)
 
         # Get min_lats not in maj_lats
-        min_lats = np.lib.arraysetops.setdiff1d(min_lats, maj_lats)
+        min_lats = np.setdiff1d(min_lats, maj_lats)
 
         # lons along major lat lines (extended slightly to avoid missing the end)
         lin_lons = np.linspace(lon_min, lon_max + Dlon / 5.0, max(self._x_size, self._y_size) // 5)
